<?php

namespace Drupal\mcapi_bursers\Plugin\TransactionRelative;

use Drupal\mcapi\Plugin\TransactionRelativeInterface;
use Drupal\mcapi\Entity\TransactionInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Database\Query\Condition;

/**
 * Defines a nominated user relative to wallets in a transaction.
 *
 * @TransactionRelative(
 *   id = "burser",
 *   label = @Translation("Burser"),
 *   description = @Translation("A user nominated to pay in to the payer wallet or payout of the payee wallet")
 * )
 */
class Burser extends PluginBase implements TransactionRelativeInterface {

  /**
   * {@inheritdoc}
   */
  public function isRelative(TransactionInterface $transaction, AccountInterface $account) : bool {
    $targets = $this->getUsers($transaction);
    return in_array($account->id(), $targets);
  }

  public function queryCondition(SelectInterface $query, Condition $or_group, $uid){
    $table_alias = $query->getTables()['mcapi_transaction']['alias'];
    $query->leftjoin('mcapi_wallet__bursers', 'payee_bursers', "payee_bursers.entity_id = $table_alias.payee");
    $query->leftjoin('mcapi_wallet__bursers', 'payer_bursers', "payer_bursers.entity_id = $table_alias.payer");
    $b_or_group = $query->orConditionGroup();
    $b_or_group->condition('payee_bursers.bursers_target_id', $uid)
      ->condition('payer_bursers.bursers_target_id', $uid);
    $or_group->condition($b_or_group);
  }

  public function queryConditionIndex(SelectInterface $query, Condition $or_group, $uid) {
    $table_alias = $query->getTables()['mcapi_transactions_index']['alias'];
    $query->leftjoin('mcapi_wallet__bursers', 'bursers_w1', "bursers_w1.entity_id = bursers_w1.wallet_id");
    $query->leftjoin('mcapi_wallet__bursers', 'bursers_w2', "bursers_w2.entity_id = burser_w2.partner_id");
    $b_or_group = $query->orConditionGroup();
    $b_or_group->condition('bursers_w1.bursers_target_id', $uid)
      ->condition('bursers_w2.bursers_target_id', $uid);
    $or_group->condition($b_or_group);
  }

  /**
   * Get the users who have bursor power over one of the wallets in the transaction
   *
   * @param TransactionInterface $transaction
   * @return int[]
   *   The ids of the users.
   */
  public function getUsers(TransactionInterface $transaction) : array {
    $payer_wallet = $transaction->payer->entity;
    $payee_wallet = $transaction->payee->entity;
    $bursers = [];
    // @todo would be nice to refactor this, considering how the payers and payees fields are used elsewhere
    foreach ([$payer_wallet, $payee_wallet] as $wallet) {
      foreach($wallet->bursers->referencedEntities() as $user) {
        $bursers[] = $user->id();
      }
    }
    return $bursers;
  }

}
