<?php

namespace Drupal\mcapi_forms;

use Drupal\mcapi\Form\TransactionForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\Entity\EntityFormMode;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Render\Element;

/**
 * Form builder for transaction formed designed via the UI.
 */
class FirstPartyTransactionForm extends TransactionForm {

  protected function getSettings($mode = NULL) {
    if (!$mode) {
      $mode = $this->getOperation();
    }
    return EntityFormMode::load('mcapi_transaction.'.$mode)->getThirdPartySettings('mcapi_forms');
  }

  /**
   * {@inheritdoc}
   */
  public function form(array $form, FormStateInterface $form_state) {
    $settings = $this->getSettings();
    if ($settings['incoming']) {
      $form_state->set('transaction_direction', static::DIR_INCOMING);
    }
    else {
      $form_state->set('transaction_direction', static::DIR_OUTGOING);
    }
    $form = parent::form($form, $form_state);

    // If a field is not in the template then set its access to FALSE so it won't fail validation.
    foreach (Element::children($form) as $key) {
      if (!$this->templateHasField($key)) {
        // hopefully the missing field will have a default value!
        $form[$key]['#access'] = FALSE;
      }
    }
    $form['#twig_template'] = str_replace(["\\r\\n","\\n","\\r"], "<br/>", $settings['experience_twig']);
    $form['#attached']['library'][] = 'mcapi_forms/display';
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function actions(array $form, FormStateInterface $form_state) {
    $actions = parent::actions($form, $form_state);
    $actions['submit']['#value'] = $this->getSettings()['experience_button'];
    // @todo check this is working.
    $form['#cache']['contexts'][] = 'user';
    return $actions;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'first_party_transaction_form';
  }

  /**
   * This is the route's _title_callback
   */
  public function title() {
    // Unfortunately This is called before the form mode is has been set in setOperation.
    $form_mode = \Drupal::routeMatch()->getRouteObject()->getDefault('_entity_form');
    $mode = substr($form_mode, strlen('mcapi_transaction.'));
    return $this->getSettings($mode)['title'];
  }

  /**
   * {@inheritdoc}
   */
  public function getEntityFromRouteMatch(RouteMatchInterface $route_match, $entity_type_id) {
    $this->request->query->add(['type' => $this->getSettings()['transaction_type']]);
    return parent::getEntityFromRouteMatch($route_match, $entity_type_id);
  }

  /**
   * {@inheritdoc}
   *
   * Fill in any wallet values not explicitly stated.
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // See https://www.drupal.org/forum/support/module-development-and-code-questions/2018-02-08/required-fields-and-not-null-database
    if (!$form_state->getValue('description')) {
       $form_state->setValue('description', ' ');
    }
    parent::validateForm($form, $form_state);
  }

  private function templateHasField(string $fieldname) {
    return preg_match(
      '/\{\{ ?'.$fieldname.' ?\}\}/',
      $this->getSettings()['experience_twig']
    );
  }

}
