<?php

namespace Drupal\mcapi_signatures\Controller;

use Drupal\mcapi\Entity\Transaction;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Add a signature from a user who may not be logged in.
 *
 * Depends on a token generated in the mcapi_signatures_mail().
 */
class RemoteSign extends ControllerBase {

  private $signatures;


  /**
   * Constructor.
   */
  public function __construct($signatures) {
    $this->signatures = $signatures;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('mcapi.signatures')
    );
  }

  /**
   * Sign the transaction and redirect to the canonical page.
   *
   * @param int $serial
   *   Serial number of a transaction.
   * @param int $uid
   *   User ID.
   * @param string $hash
   *   Hash which hopefully corresponds to #serial and $uid.
   */
  public function sign($serial, $uid, $hash) {
    if ($transaction = Transaction::loadBySerial($serial)) {
      $this->signatures->setTransaction($transaction);
      if ($this->signatures->isWaitingOn($uid)) {
        $this->signatures->sign($uid);
        $transaction->save();
        // A message is should be in the config of the sign action.
      }
      else {
        $this->messenger()->addStatus(t('The signature had already been added.'));
      }
      return $this->redirect('entity.mcapi_transaction.canonical', ['mcapi_transaction' => $serial]);
    }
    else {
      $this->messenger()->addStatus($this->t('Transaction $serial does not exist', ['serial' => $serial]), 'error');
    }
    return $this->redirect('user.page');
  }

}
