<?php

namespace Drupal\mcapi_signatures;

use Drupal\mcapi\Entity\Transaction;
use Drupal\user\Entity\User;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Routing\Access\AccessInterface;
use Drupal\Core\Site\Settings;
use Drupal\Component\Utility\Crypt;

/**
 * Determines access to routes based on login status of current user.
 * @todo inject Messenger
 */
class RemoteSignAccess implements AccessInterface  {

  /**
   * @param int $mcapi_transaction
   * @param int $uid
   * @param string $hash
   * @return AccessResult
   */
   public function access(int $mcapi_transaction, int $uid, string $hash) {
    //$mcapi_transaction is actual the serial number from the root, not yet upcast
    if ($hash == Self::generateHash($uid, $mcapi_transaction)) {
      $account = \Drupal::currentUser();
      if ($account->isAuthenticated() and $account->id() != $uid) {
        user_logout();
      }
      if ($account->isAnonymous()) {
        if ($user = User::load($uid) and $user->isActive()) {
          user_login_finalize($user);
          \Drupal::messenger()->addStatus(t('You have been logged in.'));
        }
        else {
          return AccessResult::forbidden('User is deleted or blocked.');
        }
      }
      $signatures = Transaction::load($mcapi_transaction)->signatures;
      if (isset($signatures[$uid]) and empty($signatures[$uid])) {
        return AccessResult::allowed();
      }
      return AccessResult::forbidden('Transaction does not need signature of user '.$uid);
    }
    return AccessResult::forbidden('Invalid link');
  }

  /**
  * Helper function to get a hash corresponding to a user and a transaction.
  *
  * @param int $uid
  *   A user ID.
  * @param int $serial
  *   A transaction serial number.
  */
  static function generateHash($uid, $serial) {
    return Crypt::hmacBase64(
      $uid . $serial,
      Settings::getHashSalt()
    );
  }
}
