<?php

namespace Drupal\mcapi\Entity\Access;

use Drupal\mcapi\Entity\Storage\TransactionStorageInterface;
use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityHandlerInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityTypeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines an access controller for the Currency entity.
 *
 * @see \Drupal\mcapi\Entity\Currency.
 */
class CurrencyAccessControlHandler extends EntityAccessControlHandler implements EntityHandlerInterface {

  /**
   * @var \Drupal\Core\Entity\Query\QueryInterface
   */
  protected $transactionQuery;

  /**
   * @param EntityTypeInterface $entity_type
   * @param TransactionStorageInterface $transaction_storage
   */
  public function __construct(EntityTypeInterface $entity_type, TransactionStorageInterface $transaction_storage) {
    parent::__construct($entity_type);
    $this->transactionQuery = $transaction_storage->getQuery();
  }

  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity_type.manager')->getStorage('mcapi_transaction')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    switch ($operation) {
      case 'view':
        // Page is only available once the currency has been used.
        if ($entity->firstUsed()) {
          return AccessResult::allowed()->cachePerPermissions();
        }
        return AccessResult::forbidden();// don't cache
      case 'create':
      case 'update':
        if ($account->hasPermission('configure mcapi')) {
          return AccessResult::allowed()->cachePerPermissions();
        }
        // i.e it is already saved.
        elseif ($entity->id()) {
          if ($account->id() == $entity->uid) {
            $result = AccessResult::allowed()->cachePerUser();
          }
        }
        if (!$result) {
          $result = AccessResult::forbidden("Cannot update somebody else's wallet")->cachePerUser();
        }
        break;

      case 'delete':
        // @todo inject service entity.query.config
        $count = $this->transactionQuery
          ->condition('worth.curr_id', $entity->id())
          ->count()
          ->execute();
        $result = $count ?
          AccessResult::forbidden('Cannot delete currency without first deleting transactions') :
          AccessResult::allowed();
    }
    return $result;
  }

}
