<?php

namespace Drupal\mcapi\Entity\RouteProvider;

use Drupal\mcapi\TransactionOperations;
use Drupal\Core\Entity\Routing\DefaultHtmlRouteProvider;
use Drupal\Core\Entity\EntityTypeInterface;
use Symfony\Component\Routing\Route;

/**
 * Provides routes for the transaction entity.
 */
class TransactionRouteProvider extends DefaultHtmlRouteProvider {

  /**
   * {@inheritdoc}
   */
  public function getRoutes(EntityTypeInterface $entity_type) {
    $route_collection = parent::getRoutes($entity_type);

    $route_collection
      ->get('entity.mcapi_transaction.add_form')
      ->setRequirement('_permission', 'create 3rdparty transaction');

    $route = new Route('/transaction/bill');
    $route_collection->add(
      'entity.mcapi_transaction.bill',
      $route->setDefaults(['_entity_form' => 'mcapi_transaction.bill'])
        ->setRequirement('_permission', "create bill transaction")
    );
    $route = new Route('/transaction/credit');
    $route_collection->add(
      'entity.mcapi_transaction.credit',
      $route->setDefaults(['_entity_form' => 'mcapi_transaction.credit'])
        ->setRequirement('_permission', "create credit transaction")
    );

    // Adjust the routes to load with the serial number.
    $options = $route_collection
      ->get('entity.mcapi_transaction.canonical')->getOptions();
    $options['parameters']['mcapi_transaction']['serial'] = TRUE;

    $route_collection
      ->get('entity.mcapi_transaction.canonical')
      ->setDefault('_entity_view', 'mcapi_transaction.full')
      ->setOptions($options);

    foreach (TransactionOperations::loadAllActions() as $name) {
      $route = (new Route('/transaction/{mcapi_transaction}/'.$name))
        ->setDefaults([
          '_entity_form' => 'mcapi_transaction.'.$name,
        ])
        ->setRequirement('_entity_access', 'mcapi_transaction.'.$name)
        ->setRequirement('mcapi_transaction', '\d+')
        ->setOptions($options);
      $route_collection->add('entity.mcapi_transaction.'.$name, $route);
    }
    return $route_collection;
  }

}
