<?php

namespace Drupal\mcapi\Plugin\TransactionRelative;

use Drupal\mcapi\Plugin\TransactionRelativeInterface;
use Drupal\mcapi\Entity\TransactionInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Plugin\PluginBase;
use Drupal\Core\Database\Query\SelectInterface;
use Drupal\Core\Database\Query\Condition;

/**
 * Defines a payee relative to a Transaction entity.
 *
 * @TransactionRelative(
 *   id = "payee",
 *   label = @Translation("Owner of payee wallet"),
 *   description = @Translation("The owner of the payee wallet")
 * )
 */
class Payee extends PluginBase implements TransactionRelativeInterface {

  /**
   * {@inheritdoc}
   */
  public function isRelative(TransactionInterface $transaction, AccountInterface $account) : bool {
    return $transaction->payee->entity->getOwner()->id() == $account->id();
  }

  /**
   * {@inheritdoc}
   */
  public function queryCondition(SelectInterface $query, Condition $or_group, $uid) {
    $transaction_table = $query->getTables()['mcapi_transaction'];
    $this->joinWalletTofield($query, $transaction_table['alias'].'.payee');
    $or_group->condition('payee_user.uid', $uid);
  }

  /**
   * {@inheritdoc}
   */
  public function queryConditionIndex(SelectInterface $query, Condition $or_group, $uid) {
    $transaction_table = $query->getTables()['mcapi_transactions_index'];
    $this->joinWalletTofield($query, $transaction_table['alias'].'.uid1');
    $or_group->condition('payee_user.uid', $uid);
  }

  /**
   * {@inheritdoc}
   */
  private function joinWalletTofield(SelectInterface $query, $field_alias) {
    $query->join('mcapi_wallet', 'payee_wallet', "$field_alias = payee_wallet.wid");
    $query->join('mcapi_wallet__holder', 'payee_holder', 'payee_holder.entity_id = payee_wallet.wid');
    $query->join('users', 'payee_user', "payee_holder.holder_target_id = payee_user.uid");
  }

  /**
   * {@inheritdoc}
   */
  public function getUsers(TransactionInterface $transaction) : array {
    return [$transaction->payee->entity->getOwner()->id()];
  }

}
