<?php

namespace Drupal\mcapi\Plugin\Validation\Constraint;

use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Constraint;

/**
 * Checks that all the children have the same state as the parent transaction
 */
class SameStateConstraintValidator extends ConstraintValidator {

  /**
   * {@inheritdoc}
   */
  public function validate($transaction, Constraint $constraint) {
    if ($transaction->children) {
      $state = $transaction->state->target_id;
      foreach ($transaction->flatten() as $t) {
        if ($state != $t->state->target_id) {
          $this->context->addViolation(
            $constraint->sameMessage,
            ['@main_state' => $state, '@wrong_state' => $t->state->target_id]
          );
        }
      }
    }
  }
}
