<?php

namespace Drupal\mcapi\Plugin\Validation\Constraint;

use Drupal\mcapi\Entity\Wallet;
use Symfony\Component\Validator\ConstraintValidator;
use Symfony\Component\Validator\Constraint;

/**
 * Check that no two wallets have the same name.
 */
class WalletNameUniqueValidator extends ConstraintValidator {

  /**
   * {@inheritdoc}
   */
  public function validate($items, Constraint $constraint) {
    if (!$item = $items->first()) {
      return;
    }
    /** @var \Drupal\Core\Entity\EntityInterface $entity */
    $entity = $items->getEntity();

    $query = \Drupal::entityQuery('mcapi_wallet')
      ->accessCheck(FALSE)
      ->condition('name', $item->value)
      ->range(0, 1)
      ->count();

    if (!$entity->isNew()) {
      $query->condition('wid', $entity->id(), '<>');
    }
    if ($wid = $query->execute()) {
      $wallet = Wallet::load($wid);
      $this->context->addViolation(
        $constraint->message,
        ['@name' => \Drupal\Core\Render\Markup::create($item->value)]
      );
    }
  }
}
